﻿using System;
using System.Windows;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Windows.Input;

namespace WpfGoogleMapClient
{
	/// <summary>
	/// MainWindow.xaml の相互作用ロジック
	/// </summary>
	public partial class MainWindow : Window
	{
		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		public MainWindow()
		{
			string uri       = String.Format( "file://{0}index.html", AppDomain.CurrentDomain.BaseDirectory );
			this._viewModel  = new MainWindowViewModel( uri ) { Address = "新宿" };
			this.DataContext = this._viewModel;

			this.InitializeComponent();
		}

		/// <summary>
		/// 移動ボタンが押された時に発生するイベントです。
		/// </summary>
		/// <param name="sender">イベント発生元。</param>
		/// <param name="e">イベント データ。</param>
		private void OnClickGoButton( object sender, RoutedEventArgs e )
		{
			this._webBrowser.InvokeScript( "moveMap", this._viewModel.Address );
		}

		/// <summary>
		/// 現在位置ボタンが押された時に発生するイベントです。
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void OnClickGetLocation( object sender, RoutedEventArgs e )
		{
			this._webBrowser.InvokeScript( "getLocation" );
		}

		/// <summary>
		/// MainWindow の ViewModel。
		/// </summary>
		private MainWindowViewModel _viewModel;
	}

	/// <summary>
	/// MainWindow の Model と View を仲介するクラスです。
	/// </summary>
	class MainWindowViewModel
	{
		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		/// <param name="uri">ブラウザに表示するページの URI。</param>
		public MainWindowViewModel( string uri )
		{
			this.Uri = uri;
			this.MapHost = new MapHost();
		}

		/// <summary>
		///  Google Map の操作を行う JavaScript に関連付けられるオブジェクトを取得します。
		/// </summary>
		public MapHost MapHost { get; private set; }

		/// <summary>
		/// Google Map に指定する住所を取得または設定します。
		/// </summary>
		public string Address { get; set; }

		/// <summary>
		/// ブラウザに表示するページの URI を取得します。
		/// </summary>
		public string Uri { get; private set; }
	}

	/// <summary>
	/// Google Map の操作を行う JavaScript に関連付けられるクラスです。
	/// </summary>
	[ComVisible( true )]
	public class MapHost : INotifyPropertyChanged
	{
		/// <summary>
		/// Google Map 上の移動座標。
		/// </summary>
		string _location;

		/// <summary>
		/// Google Map 上の緯度・経度を示す文字列を取得または設定します。
		/// </summary>
		public string Location
		{
			get
			{
				return this._location;
			}
			set
			{
				this._location = value;
				this.NotifyPropertyChanged( "Location" );
			}
		}

		#region INotifyPropertyChanged メンバ

		/// <summary>
		/// プロパティが変更された事を通知するイベント ハンドラ。
		/// </summary>
		public event PropertyChangedEventHandler PropertyChanged;

		/// <summary>
		/// プロパティの変更を通知します。
		/// </summary>
		/// <param name="names">変更されたプロパティ名のコレクション。</param>
		protected void NotifyPropertyChanged( params string[] names )
		{
			if( this.PropertyChanged != null )
			{
				foreach( string name in names )
				{
					this.PropertyChanged( this, new PropertyChangedEventArgs( name ) );
				}
			}
		}

		#endregion
	
	}
}
